<?php

class URL {

	function URL($arg = array()) {
		$this->self = isSet($arg['-self']) ? $arg['-self'] : true;

		if (isSet($arg['-base'])) {
			$this->base = parse_url($arg['-base']);
		} elseif ($this->self) {
			$this->base = parse_url($this->_selfURL());
		} else {
			$this->base = parse_url('');
		}

		if (isSet($arg['-url'])) {
			$this->url = parse_url($arg['-url']);
		} elseif ($this->self) {
			$this->url = parse_url($this->_selfURL());
		} else {
			$this->url = parse_url('');
		}
	}

	function _selfURL() {
		$url = '';
		if (array_key_exists('SERVER_PROTOCOL', $_SERVER)) {
	   		list($protocol, $version) = explode('/', strToLower($_SERVER['SERVER_PROTOCOL']), 2);
	    	if (array_key_exists('SERVER_PORT', $_SERVER) && $_SERVER['SERVER_PORT'] == 443) $protocol = 'https';
			$url .= $protocol .'://';
		}
	    if (array_key_exists('SERVER_NAME',  $_SERVER)) $url .= $_SERVER['SERVER_NAME'];
	    if (array_key_exists('SERVER_PORT',  $_SERVER)) $url .= ':'. $_SERVER['SERVER_PORT'];
	    if (array_key_exists('SCRIPT_NAME',  $_SERVER)) $url .= $_SERVER['SCRIPT_NAME'];
	    if (array_key_exists('PATH_INFO',    $_SERVER)) $url .= $_SERVER['PATH_INFO'];
	    if (array_key_exists('QUERY_STRING', $_SERVER)) $url .= '?'. $_SERVER['QUERY_STRING'];

		return $url;
	}

	function _setFactory($name, $value = '') {
		$this->url[$name] = $value;
		return $this->_getfactory($name);
	}

	function _getFactory($name) {
		if ($name == 'query') return isSet($this->url[$name]) ? $this->url[$name] : '';

		if (isSet($this->base[$name])) {
			if (! isSet($this->url[$name])) $this->url[$name] = $this->base[$name];
			elseif (empty($this->url[$name])) $this->url[$name] = $this->base[$name];
		}
		return isSet($this->url[$name]) ? $this->url[$name] : '';
	}

	function setScheme   ($value = '') { return $this->_setFactory('scheme', strToLower($value)); }
	function getScheme   ()            { return $this->_getFactory('scheme'); }
	function setUser     ($value = '') { return $this->_setFactory('user', strToLower($value)); }
	function getUser     ()            { return $this->_getFactory('user'); }
	function setPass     ($value = '') { return $this->_setFactory('pass', strToLower($value)); }
	function getPass     ()            { return $this->_getFactory('pass'); }
	function setHost     ($value = '') { return $this->_setFactory('host', strToLower($value)); }
	function getHost     ()            { return $this->_getFactory('host'); }
	function setPort     ($value = '') { return $this->_setFactory('port', strToLower($value)); }
	function setPath     ($value = '') { return $this->_setFactory('path', $value); }
	function getPath     ()            { return $this->_getFactory('path');  }
	function setQuery    ($value = '') { return $this->_setFactory('query', $value); }
	function getQuery    ()            { return $this->_getFactory('query');  }
	function setFragment ($value = '') { return $this->_setFactory('fragment', $value); }
	function getFragment ()            { return $this->_getFactory('fragment'); }

	function setParam($param = array(), $overwrite = true) {
		$query = $this->getParam();
		$querynames = array_keys($query);
		foreach ($param as $name => $value) {
			if (in_array($name, $querynames) && $overwrite) {
				$query[$name] = $value;
			} elseif (in_array($name, $querynames)) {
				if (!is_array($query[$name])) $query[$name] = array($query[$name]);
				$query[$name][] = $value;
			} else {
				$query[$name] = $value;
			}
		}
		$this->setQuery($this->array2query($query));
		return $query;
	}

	function getParam() {
		return $this->query2array($this->getQuery());
	}

	function getPort() {
		$sPort = $this->_getFactory('port');
		if (empty($sPort)) {
			if ($this->getScheme() == 'http') return 80;
			elseif ($this->getScheme() == 'https') return 443;
		}
		return $sPort;
	}

	// Parses the array into querystring
	// @inputs: array('A' => '1', 'B' => array('2', '3'), 'C' => array('1'), 'D' => array(array('1')))
	// @outputs: A=1&B[]=2&B[]=3&C=1&D[][]=1
	function array2query($param = array()) {
		// Convert param['name'] to store_param[]['name']
		// Convert param['name'][][] to store_param[]['name[][]']
		$store_param = array();
		foreach ($param as $name => $value) {
			if (is_array($value))
				$store_param = array_merge($store_param, $this->_store_array($name, $value));
			else
				$store_param[] = array('name' => $name, 'value' => $value);
		}

		$param = array();
		foreach ($store_param as $name => $value) {
			$param[] = $value['name'] .'='. urlencode($value['value']);
		}
		return join('&', $param);
	}

	// Convert 'name' + param[N = 0] to param['name']
	// Convert 'name' + param[N > 0] to param['name'.'[N]']
	function _store_array($name, $param) {
		$store_param = array();
		if (is_array($param)) {
			foreach ($param as $key => $value) {
				if (is_array($value)) {
					$store_param = array_merge($store_param, $this->_store_array($name.'['.$key.']', $value));
				} else {
					$store_param[] = array('name' => $name.'['.$key.']', 'value' => $value);
				}
			}
		} else {
			$store_param[] = array('name' => $name, 'value' => $value);
		}
		return $store_param;
	}

	// Parses the querystring into array of variables
	// @inputs: first=value&second[]=this+works&second[]=another
	// @outputs: array('first' => 'value', 'second' => array('this+works', 'another'))
	function query2array($query = '') {
		parse_str($query, $param);
	    return $param;
	}

	function getUrl($arg = array()) {
		$arg['-full'] = isset($arg['-full']) ? $arg['-full'] : true;
		$arg['-absolute'] = isset($arg['-absolute']) ? $arg['-absolute'] : false;
		$arg['-query'] = isset($arg['-query']) ? $arg['-query'] : true;

		if (isSet($arg['-absolute']) && $arg['-absolute']) $arg['-full'] = false;
		if (isSet($arg['-base']) && $arg['-base']) $arg['-full'] = true;

		if ($arg['-full']) {
			$url = $this->getScheme() .'://';

			if     ($this->getScheme() == 'http' && $this->getPort() == 80) $url .= $this->getHost();
			elseif ($this->getScheme() == 'https' && $this->getPort() == 443) $url .= $this->getHost();
			elseif ($this->getPort()) $url .= $this->getHost() .':'. $this->getPort();

	        if (isSet($arg['-base']) && $arg['-base']) return $url;

			$url .= $this->getPath();
		} elseif ($arg['-absolute']) {
			$url = $this->getPath();
		}

		if ($arg['-query'] && $this->getQuery()) $url .= '?'. $this->getQuery();
		if ($this->getFragment()) $url .= '#'. $this->getFragment();

	    return $url;
	}

}

?>
