<?php
/**
 *
 *
 */

class SQL
{

	function prepareQuery($sKey, $query_vars)
	{
		global $_SQL;
    	$query_string = str_replace('%%','^', $_SQL[$sKey]);

		if(is_array($query_vars)) {

			if(count($query_vars) != preg_match_all('/%[sSiIfFcCqQ]/',$query_string, $match)) {
				trigger_error('unmatched number of vars and % placeholders: ' . $query_string);
			}

			// get string position for each element
			$var_pos = array();
			$curr_pos = 0;
			for($x = 0, $y = count($query_vars); $x < $y; $x++) {
				$var_pos[$x] = strpos($query_string, $match[0][$x], $curr_pos);
				$curr_pos = $var_pos[$x] + 1;
			}
			// build query from passed in variables, escape them
			for($x = count($query_vars) - 1, $y = 0; $x >= $y; $x--) {
				// escape string
				$query_vars[$x] = SQL::sql_escape($query_vars[$x]);
				if(in_array($match[0][$x],array('%S','%I','%F','%C','%Q'))) {
					// get positions of [ and ]
					$right_pos = strpos($query_string, ']', $var_pos[$x]);
					$str_slice = substr($query_string, 0, $var_pos[$x]);
					$left_pos = strrpos($str_slice,'[');
					if($right_pos === false || $left_pos === false) {
						trigger_error('missing or unmatched brackets for % placeholder(s): ' . $query_string);
					}
					if($query_vars[$x] == '') {
						// remove entire part of string
						$query_string = substr_replace($query_string, '', $left_pos, $right_pos - $left_pos + 1);
					} else {
						// remove the brackets only, and replace %S
						$query_string = substr_replace($query_string, '', $right_pos, 1);
						$query_string = substr_replace($query_string, SQL::convert_var($query_vars[$x],$match[0][$x]), $var_pos[$x], 2);
						$query_string = substr_replace($query_string, '', $left_pos, 1);
					}
				} else {
					$query_string = substr_replace($query_string, SQL::convert_var($query_vars[$x],$match[0][$x]), $var_pos[$x], 2);
				}
			}
		}

    	$query_string = str_replace('^','%',$query_string);
		return $query_string;

	}

/*======================================================================*\
    Function: sql_escape
    Purpose:  recursively escape variables/arrays for SQL use
\*======================================================================*/
	function sql_escape($v) {
		if(is_array($v)) {
			foreach($v as $element) {
				$newvar[] = SQL::sql_escape($element);
			}
			return $newvar;
		}
		return addslashes($v);
	}

/*======================================================================*\
    Function: convert_var
    Purpose:  convert a variable to the given type
	Input:    $v - the variable
			  $type - the type to convert to:
			  	%i, %I - cast to integer
				%f, %F - cast to float
				%c, %C - comma separate, cast each element to integer
				%q, %Q - quote/comma separate
\*======================================================================*/
	function convert_var($v, $type) {
		switch($type) {
			case '%s':
				// cast to string
				settype($v, 'string');
				$v = $v;
                break;
			case '%i':
			case '%I':
				// cast to integer
				settype($v, 'integer');
				break;
			case '%f':
			case '%F':
				// cast to float
				settype($v, 'float');
				break;
			case '%c':
			case '%C':
				// comma separate
				settype($v, 'array');
				for($x = 0 , $y = count($v); $x < $y; $x++) {
					// cast to integers
					settype($v[$x], 'integer');
				}
				$v = implode(',', $v);
				if($v == '') {
					// force 0, keep syntax from breaking
					$v = '0';
				}
				break;
			case '%q':
			case '%Q':
				settype($v, 'array');
				// quote comma separate
				$v = "'" . implode("','",$v) . "'";
				break;
		}
		return $v;
	}
}

?>